=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-09-22 15:23:10 +0000
+++ drizzled/Makefile.am	2009-09-23 17:01:21 +0000
@@ -18,10 +18,16 @@
 
 DTRACEFILES = handler.o \
 	      filesort.o \
+	      drizzled.o \
+	      session.o \
 	      sql_delete.o \
 	      sql_insert.o \
 	      sql_select.o \
-	      sql_update.o
+	      sql_update.o \
+	      sql_parse.o \
+	      statement/delete.o \
+	      statement/insert.o \
+	      statement/insert_select.o
 
 drizzledincludedir = ${includedir}/drizzled
 nobase_dist_drizzledinclude_HEADERS = \
@@ -699,10 +705,6 @@
 		util/convert.cc \
 		xid.cc
 
-if HAVE_DTRACE
-drizzled_SOURCES += probes.d
-endif
-
 libserialutil_la_SOURCES = db.cc table_proto_write.cc
 libserialutil_la_CXXFLAGS= ${AM_CXXFLAGS} ${PROTOSKIP_WARNINGS}
 
@@ -730,7 +732,7 @@
 BUILT_SOURCES = $(BUILT_MAINT_SRC) \
 		symbol_hash.h \
 		function_hash.h \
-		$(PCHHEADERS)
+		$(PCHHEADERS) 
 
 EXTRA_DIST = \
 		plugin/config.h.in \
@@ -750,16 +752,29 @@
 		$(PCHHEADERS) \
 		$(nodist_drizzled_SOURCES)
 
+if DTRACE_NEEDS_OBJECTS
+drizzled_SOURCES += probes.d
+endif
+
+if HAVE_DTRACE
+BUILT_SOURCES += generated_probes.h
+CLEANFILES += generated_probes.h
+endif
+
+
 DISTCLEANFILES = $(EXTRA_PROGRAMS) \
 		 $(BUILT_MAINT_SRC)
 
 MAINTAINERCLEANFILES = $(BUILT_MAINT_SRC)
 
-probes.h: probes.d
-	$(DTRACE) $(DTRACEFLAGS) -h -s probes.d
-	mv probes.h probes.h.bak
-	sed "s/#include <unistd.h>//g" probes.h.bak > probes.h
-	rm probes.h.bak
+generated_probes.h : probes.d
+	$(DTRACE) $(DTRACEFLAGS) -h -s probes.d -o generated_probes.h
+	mv generated_probes.h generated_probes.h.bak
+	sed "s/#include <unistd.h>//g" generated_probes.h.bak > generated_probes.h
+	rm generated_probes.h.bak
+	sed -e 's,void \*,const void \*,g' generated_probes.h | \
+		sed -e 's,char \*,const char \*,g' | tr '\t' ' ' > dtrace_probes.tmp
+	mv dtrace_probes.tmp generated_probes.h
 
 
 SUFFIXES = .d .gch .gperf .stamp-h

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-09-22 07:35:28 +0000
+++ drizzled/drizzled.cc	2009-09-23 17:01:21 +0000
@@ -45,6 +45,7 @@
 #include "drizzled/temporal_format.h" /* For init_temporal_formats() */
 #include "drizzled/slot/listen.h"
 #include "drizzled/plugin/client.h"
+#include "drizzled/probes.h"
 
 #include <google/protobuf/stubs/common.h>
 
@@ -457,6 +458,7 @@
     tmp= *it;
     tmp->killed= Session::KILL_CONNECTION;
     tmp->scheduler->killSession(tmp);
+    DRIZZLE_CONNECTION_DONE(tmp->thread_id);
     if (tmp->mysys_var)
     {
       tmp->mysys_var->abort=1;
@@ -767,6 +769,7 @@
   {
     statistic_increment(killed_threads, &LOCK_status);
     session->scheduler->killSessionNow(session);
+    DRIZZLE_CONNECTION_DONE(session->thread_id);
   }
   return;
 }

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-08-20 20:29:18 +0000
+++ drizzled/filesort.cc	2009-09-13 01:09:19 +0000
@@ -114,7 +114,7 @@
   TableList *tab= table->pos_in_table_list;
   Item_subselect *subselect= tab ? tab->containing_subselect() : 0;
 
-  DRIZZLE_FILESORT_START();
+  DRIZZLE_FILESORT_START(table->s->db.str, table->s->table_name.str);
 
   /*
    Release InnoDB's adaptive hash index latch (if holding) before
@@ -324,8 +324,8 @@
 		  (uint32_t) records, &LOCK_status);
   *examined_rows= param.examined_rows;
   memcpy(&table->sort, &table_sort, sizeof(filesort_info_st));
-  DRIZZLE_FILESORT_END();
-  return(error ? HA_POS_ERROR : records);
+  DRIZZLE_FILESORT_DONE(error, records);
+  return (error ? HA_POS_ERROR : records);
 } /* filesort */
 
 

=== modified file 'drizzled/handler.cc'
--- drizzled/handler.cc	2009-08-27 18:06:03 +0000
+++ drizzled/handler.cc	2009-09-23 17:01:21 +0000
@@ -2693,9 +2693,9 @@
     We cache the table flags if the locking succeeded. Otherwise, we
     keep them as they were when they were fetched in ha_open().
   */
-  DRIZZLE_EXTERNAL_LOCK(lock_type);
 
   int error= external_lock(session, lock_type);
+
   if (error == 0)
     cached_table_flags= table_flags();
   return error;
@@ -2726,7 +2726,6 @@
 int handler::ha_write_row(unsigned char *buf)
 {
   int error;
-  DRIZZLE_INSERT_ROW_START();
 
   /* 
    * If we have a timestamp column, update it to the current time 
@@ -2740,12 +2739,13 @@
   mark_trx_read_write();
 
   if (unlikely(error= write_row(buf)))
+  {
     return error;
+  }
 
   if (unlikely(log_row_for_replication(table, 0, buf)))
     return HA_ERR_RBR_LOGGING_FAILED; /* purecov: inspected */
 
-  DRIZZLE_INSERT_ROW_END();
   return 0;
 }
 
@@ -2763,7 +2763,9 @@
   mark_trx_read_write();
 
   if (unlikely(error= update_row(old_data, new_data)))
+  {
     return error;
+  }
 
   if (unlikely(log_row_for_replication(table, old_data, new_data)))
     return HA_ERR_RBR_LOGGING_FAILED;

=== modified file 'drizzled/probes.d'
--- drizzled/probes.d	2008-11-01 00:46:20 +0000
+++ drizzled/probes.d	2009-09-14 20:44:27 +0000
@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,18 +17,107 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/*
+  The actual probe names in DTrace scripts will replace '__' by '-'. Thus
+  insert__row__start will be insert-row-start.
+
+  Recommendations for adding new probes:
+
+  - each probe should have the minimal set of arguments required to
+  unambiguously identify the context in which the probe fires. Redundant
+  probes (i.e. the ones that can be obtained in user scripts from previous
+  probes' arguments or otherwise) may be added for convenience.
+
+  - try to avoid computationally expensive probe arguments. If impossible,
+  use *_ENABLED() macros to check if the probe is activated before
+  performing expensive calculations for a probe argument.
+
+  - all *-done probes should have a status argument wherever applicable to make
+  it possible for user scripts to figure out whether the completed operation
+  was successful or not.
+  
+  - for all status arguments, a non-zero value should be returned on error or
+  failure, 0 should be returned on success.
+*/
+
 provider drizzle {
-probe external_lock(int);
-probe insert_row_start();
-probe insert_row_end();
-probe filesort_start();
-probe filesort_end();
-probe delete_start();
-probe delete_end();
-probe insert_start();
-probe insert_end();
-probe select_start();
-probe select_end();
-probe update_start();
-probe update_end();
+  
+  /* The following ones fire when creating or closing a client connection */
+  probe connection__start(unsigned long conn_id);
+  probe connection__done(unsigned long conn_id);
+
+  /*
+   * Fire at the start/end of any client command processing (including SQL
+   * queries).
+  */
+  probe command__start(unsigned long conn_id, int command);
+  probe command__done(int status);
+  
+  /*
+   * The following probes fire at the start/end of any SQL query processing,
+   * respectively.
+   *
+   * query_start() has a lot of parameters that can be used to pick up
+   * parameters for a lot of other probes here.  For simplicity reasons we also
+   * add the query string to most other DTrace probes as well. Hostname is
+   * either the hostname or the IP address of the Drizzle client.
+   */
+  probe query__start(const char *query,
+                     unsigned long conn_id,
+                     const char *db_name);
+  probe query__done(int status); 
+
+  /* Fire at the start/end of SQL query parsing */
+  probe query__parse__start(const char *query);
+  probe query__parse__done(int status);
+
+  /*
+   * This probe fires when the actual query execution starts
+   */
+  probe query__exec__start(const char *query,
+                           unsigned long connid,
+                           const char *db_name);
+  probe query__exec__done(int status);
+
+  /* These probes fire when performing write operations towards any handler */
+  probe insert__row__start(const char *db, const char *table);
+  probe insert__row__done(int status);
+  probe update__row__start(const char *db, const char *table);
+  probe update__row__done(int status);
+  probe delete__row__start(const char *db, const char *table);
+  probe delete__row__done(int status);
+
+  /*
+   * These probes fire when calling external_lock for any handler
+   * depending on the lock type being acquired or released.
+   */
+  probe handler__rdlock__start(const char *db, const char *table);
+  probe handler__wrlock__start(const char *db, const char *table);
+  probe handler__unlock__start(const char *db, const char *table);
+  probe handler__rdlock__done(int status);
+  probe handler__wrlock__done(int status);
+  probe handler__unlock__done(int status);
+  
+  /*
+   * These probes fire when a filesort activity happens in a query.
+   */
+  probe filesort__start(const char *db, const char *table);
+  probe filesort__done(int status, unsigned long rows);
+  /*
+   * The query types SELECT, INSERT, INSERT AS SELECT, UPDATE, DELETE
+   * are all probed.
+   * The start probe always contains the query text.
+   */
+  probe select__start(const char *query);
+  probe select__done(int status, unsigned long rows);
+  probe insert__start(const char *query);
+  probe insert__done(int status, unsigned long rows);
+  probe insert__select__start(const char *query);
+  probe insert__select__done(int status, unsigned long rows);
+  probe update__start(const char *query);
+  probe update__done(int status,
+                     unsigned long rowsmatches, unsigned long rowschanged);
+  probe delete__start(const char *query);
+  probe delete__done(int status, unsigned long rows);
+
 };

=== removed file 'drizzled/probes.h'
--- drizzled/probes.h	2009-05-21 17:05:14 +0000
+++ drizzled/probes.h	1970-01-01 00:00:00 +0000
@@ -1,250 +0,0 @@
-/*
- * Generated by dtrace(1M).
- */
-
-#ifndef	_PROBES_H
-#define	_PROBES_H
-
-
-
-#ifdef	__cplusplus
-extern "C" {
-#endif
-
-#if _DTRACE_VERSION
-
-#define	DRIZZLE_DELETE_END() \
-	__dtrace_drizzle___delete_end()
-#ifndef	__sparc
-#define	DRIZZLE_DELETE_END_ENABLED() \
-	__dtraceenabled_drizzle___delete_end()
-#else
-#define	DRIZZLE_DELETE_END_ENABLED() \
-	__dtraceenabled_drizzle___delete_end(0)
-#endif
-#define	DRIZZLE_DELETE_START() \
-	__dtrace_drizzle___delete_start()
-#ifndef	__sparc
-#define	DRIZZLE_DELETE_START_ENABLED() \
-	__dtraceenabled_drizzle___delete_start()
-#else
-#define	DRIZZLE_DELETE_START_ENABLED() \
-	__dtraceenabled_drizzle___delete_start(0)
-#endif
-#define	DRIZZLE_EXTERNAL_LOCK(arg0) \
-	__dtrace_drizzle___external_lock(arg0)
-#ifndef	__sparc
-#define	DRIZZLE_EXTERNAL_LOCK_ENABLED() \
-	__dtraceenabled_drizzle___external_lock()
-#else
-#define	DRIZZLE_EXTERNAL_LOCK_ENABLED() \
-	__dtraceenabled_drizzle___external_lock(0)
-#endif
-#define	DRIZZLE_FILESORT_END() \
-	__dtrace_drizzle___filesort_end()
-#ifndef	__sparc
-#define	DRIZZLE_FILESORT_END_ENABLED() \
-	__dtraceenabled_drizzle___filesort_end()
-#else
-#define	DRIZZLE_FILESORT_END_ENABLED() \
-	__dtraceenabled_drizzle___filesort_end(0)
-#endif
-#define	DRIZZLE_FILESORT_START() \
-	__dtrace_drizzle___filesort_start()
-#ifndef	__sparc
-#define	DRIZZLE_FILESORT_START_ENABLED() \
-	__dtraceenabled_drizzle___filesort_start()
-#else
-#define	DRIZZLE_FILESORT_START_ENABLED() \
-	__dtraceenabled_drizzle___filesort_start(0)
-#endif
-#define	DRIZZLE_INSERT_END() \
-	__dtrace_drizzle___insert_end()
-#ifndef	__sparc
-#define	DRIZZLE_INSERT_END_ENABLED() \
-	__dtraceenabled_drizzle___insert_end()
-#else
-#define	DRIZZLE_INSERT_END_ENABLED() \
-	__dtraceenabled_drizzle___insert_end(0)
-#endif
-#define	DRIZZLE_INSERT_ROW_END() \
-	__dtrace_drizzle___insert_row_end()
-#ifndef	__sparc
-#define	DRIZZLE_INSERT_ROW_END_ENABLED() \
-	__dtraceenabled_drizzle___insert_row_end()
-#else
-#define	DRIZZLE_INSERT_ROW_END_ENABLED() \
-	__dtraceenabled_drizzle___insert_row_end(0)
-#endif
-#define	DRIZZLE_INSERT_ROW_START() \
-	__dtrace_drizzle___insert_row_start()
-#ifndef	__sparc
-#define	DRIZZLE_INSERT_ROW_START_ENABLED() \
-	__dtraceenabled_drizzle___insert_row_start()
-#else
-#define	DRIZZLE_INSERT_ROW_START_ENABLED() \
-	__dtraceenabled_drizzle___insert_row_start(0)
-#endif
-#define	DRIZZLE_INSERT_START() \
-	__dtrace_drizzle___insert_start()
-#ifndef	__sparc
-#define	DRIZZLE_INSERT_START_ENABLED() \
-	__dtraceenabled_drizzle___insert_start()
-#else
-#define	DRIZZLE_INSERT_START_ENABLED() \
-	__dtraceenabled_drizzle___insert_start(0)
-#endif
-#define	DRIZZLE_SELECT_END() \
-	__dtrace_drizzle___select_end()
-#ifndef	__sparc
-#define	DRIZZLE_SELECT_END_ENABLED() \
-	__dtraceenabled_drizzle___select_end()
-#else
-#define	DRIZZLE_SELECT_END_ENABLED() \
-	__dtraceenabled_drizzle___select_end(0)
-#endif
-#define	DRIZZLE_SELECT_START() \
-	__dtrace_drizzle___select_start()
-#ifndef	__sparc
-#define	DRIZZLE_SELECT_START_ENABLED() \
-	__dtraceenabled_drizzle___select_start()
-#else
-#define	DRIZZLE_SELECT_START_ENABLED() \
-	__dtraceenabled_drizzle___select_start(0)
-#endif
-#define	DRIZZLE_UPDATE_END() \
-	__dtrace_drizzle___update_end()
-#ifndef	__sparc
-#define	DRIZZLE_UPDATE_END_ENABLED() \
-	__dtraceenabled_drizzle___update_end()
-#else
-#define	DRIZZLE_UPDATE_END_ENABLED() \
-	__dtraceenabled_drizzle___update_end(0)
-#endif
-#define	DRIZZLE_UPDATE_START() \
-	__dtrace_drizzle___update_start()
-#ifndef	__sparc
-#define	DRIZZLE_UPDATE_START_ENABLED() \
-	__dtraceenabled_drizzle___update_start()
-#else
-#define	DRIZZLE_UPDATE_START_ENABLED() \
-	__dtraceenabled_drizzle___update_start(0)
-#endif
-
-
-extern void __dtrace_drizzle___delete_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___delete_end(void);
-#else
-extern int __dtraceenabled_drizzle___delete_end(long);
-#endif
-extern void __dtrace_drizzle___delete_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___delete_start(void);
-#else
-extern int __dtraceenabled_drizzle___delete_start(long);
-#endif
-extern void __dtrace_drizzle___external_lock(int);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___external_lock(void);
-#else
-extern int __dtraceenabled_drizzle___external_lock(long);
-#endif
-extern void __dtrace_drizzle___filesort_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___filesort_end(void);
-#else
-extern int __dtraceenabled_drizzle___filesort_end(long);
-#endif
-extern void __dtrace_drizzle___filesort_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___filesort_start(void);
-#else
-extern int __dtraceenabled_drizzle___filesort_start(long);
-#endif
-extern void __dtrace_drizzle___insert_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___insert_end(void);
-#else
-extern int __dtraceenabled_drizzle___insert_end(long);
-#endif
-extern void __dtrace_drizzle___insert_row_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___insert_row_end(void);
-#else
-extern int __dtraceenabled_drizzle___insert_row_end(long);
-#endif
-extern void __dtrace_drizzle___insert_row_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___insert_row_start(void);
-#else
-extern int __dtraceenabled_drizzle___insert_row_start(long);
-#endif
-extern void __dtrace_drizzle___insert_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___insert_start(void);
-#else
-extern int __dtraceenabled_drizzle___insert_start(long);
-#endif
-extern void __dtrace_drizzle___select_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___select_end(void);
-#else
-extern int __dtraceenabled_drizzle___select_end(long);
-#endif
-extern void __dtrace_drizzle___select_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___select_start(void);
-#else
-extern int __dtraceenabled_drizzle___select_start(long);
-#endif
-extern void __dtrace_drizzle___update_end(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___update_end(void);
-#else
-extern int __dtraceenabled_drizzle___update_end(long);
-#endif
-extern void __dtrace_drizzle___update_start(void);
-#ifndef	__sparc
-extern int __dtraceenabled_drizzle___update_start(void);
-#else
-extern int __dtraceenabled_drizzle___update_start(long);
-#endif
-
-#else
-
-#define	DRIZZLE_DELETE_END()
-#define	DRIZZLE_DELETE_END_ENABLED() (0)
-#define	DRIZZLE_DELETE_START()
-#define	DRIZZLE_DELETE_START_ENABLED() (0)
-#define	DRIZZLE_EXTERNAL_LOCK(arg0)
-#define	DRIZZLE_EXTERNAL_LOCK_ENABLED() (0)
-#define	DRIZZLE_FILESORT_END()
-#define	DRIZZLE_FILESORT_END_ENABLED() (0)
-#define	DRIZZLE_FILESORT_START()
-#define	DRIZZLE_FILESORT_START_ENABLED() (0)
-#define	DRIZZLE_INSERT_END()
-#define	DRIZZLE_INSERT_END_ENABLED() (0)
-#define	DRIZZLE_INSERT_ROW_END()
-#define	DRIZZLE_INSERT_ROW_END_ENABLED() (0)
-#define	DRIZZLE_INSERT_ROW_START()
-#define	DRIZZLE_INSERT_ROW_START_ENABLED() (0)
-#define	DRIZZLE_INSERT_START()
-#define	DRIZZLE_INSERT_START_ENABLED() (0)
-#define	DRIZZLE_SELECT_END()
-#define	DRIZZLE_SELECT_END_ENABLED() (0)
-#define	DRIZZLE_SELECT_START()
-#define	DRIZZLE_SELECT_START_ENABLED() (0)
-#define	DRIZZLE_UPDATE_END()
-#define	DRIZZLE_UPDATE_END_ENABLED() (0)
-#define	DRIZZLE_UPDATE_START()
-#define	DRIZZLE_UPDATE_START_ENABLED() (0)
-
-#endif
-
-
-#ifdef	__cplusplus
-}
-#endif
-
-#endif	/* _PROBES_H */

=== added file 'drizzled/probes.h'
--- drizzled/probes.h	1970-01-01 00:00:00 +0000
+++ drizzled/probes.h	2009-09-14 20:12:21 +0000
@@ -0,0 +1,97 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_PROBES_H
+#define DRIZZLED_PROBES_H
+
+#ifdef HAVE_DTRACE
+#include "drizzled/generated_probes.h"
+#else
+#define DRIZZLE_COMMAND_DONE(arg0)
+#define DRIZZLE_COMMAND_DONE_ENABLED() (0)
+#define DRIZZLE_COMMAND_START(arg0, arg1)
+#define DRIZZLE_COMMAND_START_ENABLED() (0)
+#define DRIZZLE_CONNECTION_DONE(arg0)
+#define DRIZZLE_CONNECTION_DONE_ENABLED() (0)
+#define DRIZZLE_CONNECTION_START(arg0)
+#define DRIZZLE_CONNECTION_START_ENABLED() (0)
+#define DRIZZLE_DELETE_DONE(arg0, arg1)
+#define DRIZZLE_DELETE_DONE_ENABLED() (0)
+#define DRIZZLE_DELETE_ROW_DONE(arg0)
+#define DRIZZLE_DELETE_ROW_DONE_ENABLED() (0)
+#define DRIZZLE_DELETE_ROW_START(arg0, arg1)
+#define DRIZZLE_DELETE_ROW_START_ENABLED() (0)
+#define DRIZZLE_DELETE_START(arg0)
+#define DRIZZLE_DELETE_START_ENABLED() (0)
+#define DRIZZLE_FILESORT_DONE(arg0, arg1)
+#define DRIZZLE_FILESORT_DONE_ENABLED() (0)
+#define DRIZZLE_FILESORT_START(arg0, arg1)
+#define DRIZZLE_FILESORT_START_ENABLED() (0)
+#define DRIZZLE_HANDLER_RDLOCK_DONE(arg0)
+#define DRIZZLE_HANDLER_RDLOCK_DONE_ENABLED() (0)
+#define DRIZZLE_HANDLER_RDLOCK_START(arg0, arg1)
+#define DRIZZLE_HANDLER_RDLOCK_START_ENABLED() (0)
+#define DRIZZLE_HANDLER_UNLOCK_DONE(arg0)
+#define DRIZZLE_HANDLER_UNLOCK_DONE_ENABLED() (0)
+#define DRIZZLE_HANDLER_UNLOCK_START(arg0, arg1)
+#define DRIZZLE_HANDLER_UNLOCK_START_ENABLED() (0)
+#define DRIZZLE_HANDLER_WRLOCK_DONE(arg0)
+#define DRIZZLE_HANDLER_WRLOCK_DONE_ENABLED() (0)
+#define DRIZZLE_HANDLER_WRLOCK_START(arg0, arg1)
+#define DRIZZLE_HANDLER_WRLOCK_START_ENABLED() (0)
+#define DRIZZLE_INSERT_DONE(arg0, arg1)
+#define DRIZZLE_INSERT_DONE_ENABLED() (0)
+#define DRIZZLE_INSERT_ROW_DONE(arg0)
+#define DRIZZLE_INSERT_ROW_DONE_ENABLED() (0)
+#define DRIZZLE_INSERT_ROW_START(arg0, arg1)
+#define DRIZZLE_INSERT_ROW_START_ENABLED() (0)
+#define DRIZZLE_INSERT_SELECT_DONE(arg0, arg1)
+#define DRIZZLE_INSERT_SELECT_DONE_ENABLED() (0)
+#define DRIZZLE_INSERT_SELECT_START(arg0)
+#define DRIZZLE_INSERT_SELECT_START_ENABLED() (0)
+#define DRIZZLE_INSERT_START(arg0)
+#define DRIZZLE_INSERT_START_ENABLED() (0)
+#define DRIZZLE_QUERY_DONE(arg0)
+#define DRIZZLE_QUERY_DONE_ENABLED() (0)
+#define DRIZZLE_QUERY_EXEC_DONE(arg0)
+#define DRIZZLE_QUERY_EXEC_DONE_ENABLED() (0)
+#define DRIZZLE_QUERY_EXEC_START(arg0, arg1, arg2)
+#define DRIZZLE_QUERY_EXEC_START_ENABLED() (0)
+#define DRIZZLE_QUERY_PARSE_DONE(arg0)
+#define DRIZZLE_QUERY_PARSE_DONE_ENABLED() (0)
+#define DRIZZLE_QUERY_PARSE_START(arg0)
+#define DRIZZLE_QUERY_PARSE_START_ENABLED() (0)
+#define DRIZZLE_QUERY_START(arg0, arg1, arg2)
+#define DRIZZLE_QUERY_START_ENABLED() (0)
+#define DRIZZLE_SELECT_DONE(arg0, arg1)
+#define DRIZZLE_SELECT_DONE_ENABLED() (0)
+#define DRIZZLE_SELECT_START(arg0)
+#define DRIZZLE_SELECT_START_ENABLED() (0)
+#define DRIZZLE_UPDATE_DONE(arg0, arg1, arg2)
+#define DRIZZLE_UPDATE_DONE_ENABLED() (0)
+#define DRIZZLE_UPDATE_ROW_DONE(arg0)
+#define DRIZZLE_UPDATE_ROW_DONE_ENABLED() (0)
+#define DRIZZLE_UPDATE_ROW_START(arg0, arg1)
+#define DRIZZLE_UPDATE_ROW_START_ENABLED() (0)
+#define DRIZZLE_UPDATE_START(arg0)
+#define DRIZZLE_UPDATE_START_ENABLED() (0)
+#endif
+
+#endif /* DRIZZLED_PROBES_H */

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2009-09-22 07:35:28 +0000
+++ drizzled/session.cc	2009-09-23 17:01:21 +0000
@@ -38,6 +38,7 @@
 #include <drizzled/show.h>
 #include <drizzled/scheduling.h>
 #include <drizzled/plugin/client.h>
+#include "drizzled/probes.h"
 
 #include <algorithm>
 
@@ -480,6 +481,7 @@
   if (state_to_set != Session::KILL_QUERY)
   {
     scheduler->killSession(this);
+    DRIZZLE_CONNECTION_DONE(thread_id);
   }
   if (mysys_var)
   {
@@ -619,6 +621,7 @@
 
   if (scheduler->addSession(this))
   {
+    DRIZZLE_CONNECTION_START(thread_id);
     char error_message_buff[DRIZZLE_ERRMSG_SIZE];
 
     killed= Session::KILL_CONNECTION;

=== modified file 'drizzled/sql_delete.cc'
--- drizzled/sql_delete.cc	2009-08-06 16:51:53 +0000
+++ drizzled/sql_delete.cc	2009-09-17 00:08:20 +0000
@@ -25,6 +25,7 @@
 #include <drizzled/sql_parse.h>
 #include <drizzled/sql_base.h>
 #include <drizzled/lock.h>
+#include "drizzled/probes.h"
 
 /**
   Implement DELETE SQL word.
@@ -51,9 +52,11 @@
   Select_Lex   *select_lex= &session->lex->select_lex;
   Session::killed_state killed_status= Session::NOT_KILLED;
 
-
   if (session->openTablesLock(table_list))
+  {
+    DRIZZLE_DELETE_DONE(1, 0);
     return true;
+  }
 
   table= table_list->table;
   assert(table);
@@ -162,14 +165,14 @@
     delete select;
     free_underlaid_joins(session, select_lex);
     session->row_count_func= 0;
-    DRIZZLE_DELETE_END();
+    DRIZZLE_DELETE_DONE(0, 0);
     session->my_ok((ha_rows) session->row_count_func);
     /*
       We don't need to call reset_auto_increment in this case, because
       mysql_truncate always gives a NULL conds argument, hence we never
       get here.
     */
-    return(0);				// Nothing to delete
+    return 0; // Nothing to delete
   }
 
   /* If running in safe sql mode, don't allow updates without keys */
@@ -322,17 +325,17 @@
   assert(transactional_table || !deleted || session->transaction.stmt.modified_non_trans_table);
   free_underlaid_joins(session, select_lex);
 
-  DRIZZLE_DELETE_END();
+  DRIZZLE_DELETE_DONE((error >= 0 || session->is_error()), deleted);
   if (error < 0 || (session->lex->ignore && !session->is_fatal_error))
   {
     session->row_count_func= deleted;
     session->my_ok((ha_rows) session->row_count_func);
   }
-  return(error >= 0 || session->is_error());
+  return (error >= 0 || session->is_error());
 
 err:
-  DRIZZLE_DELETE_END();
-  return(true);
+  DRIZZLE_DELETE_DONE(1, 0);
+  return true;
 }
 
 

=== modified file 'drizzled/sql_insert.cc'
--- drizzled/sql_insert.cc	2009-09-22 07:35:28 +0000
+++ drizzled/sql_insert.cc	2009-09-23 17:01:21 +0000
@@ -240,7 +240,10 @@
                     values_list.elements > 1);
 
   if (session->openTablesLock(table_list))
+  {
+    DRIZZLE_INSERT_DONE(1, 0);
     return true;
+  }
 
   lock_type= table_list->lock_type;
 
@@ -466,8 +469,8 @@
                    info.copied + info.deleted + info.touched, id, buff);
   }
   session->abort_on_warning= 0;
-  DRIZZLE_INSERT_END();
-  return(false);
+  DRIZZLE_INSERT_DONE(0, session->row_count_func);
+  return false;
 
 abort:
   if (table != NULL)
@@ -475,8 +478,8 @@
   if (!joins_freed)
     free_underlaid_joins(session, &session->lex->select_lex);
   session->abort_on_warning= 0;
-  DRIZZLE_INSERT_END();
-  return(true);
+  DRIZZLE_INSERT_DONE(1, 0);
+  return true;
 }
 
 
@@ -1336,7 +1339,8 @@
   if (error)
   {
     table->file->print_error(error,MYF(0));
-    return(1);
+    DRIZZLE_INSERT_SELECT_DONE(error, 0);
+    return 1;
   }
   char buff[160];
   if (info.ignore)
@@ -1354,7 +1358,8 @@
      (info.copied ? autoinc_value_of_last_inserted_row : 0));
   session->my_ok((ulong) session->row_count_func,
                  info.copied + info.deleted + info.touched, id, buff);
-  return(0);
+  DRIZZLE_INSERT_SELECT_DONE(0, session->row_count_func);
+  return 0;
 }
 
 void select_insert::abort() {
@@ -1393,6 +1398,11 @@
     table->file->ha_release_auto_increment();
   }
 
+  if (DRIZZLE_INSERT_SELECT_DONE_ENABLED())
+  {
+    DRIZZLE_INSERT_SELECT_DONE(0, info.copied + info.deleted + info.updated);
+  }
+
   return;
 }
 

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2009-09-22 07:35:28 +0000
+++ drizzled/sql_parse.cc	2009-09-23 17:01:21 +0000
@@ -36,6 +36,7 @@
 #include <drizzled/select_send.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/statement.h>
+#include "drizzled/probes.h"
 
 #include <bitset>
 #include <algorithm>
@@ -167,7 +168,10 @@
   bool error= 0;
   Query_id &query_id= Query_id::get_query_id();
 
-  session->command=command;
+  DRIZZLE_COMMAND_START(session->thread_id,
+                        command);
+
+  session->command= command;
   session->lex->sql_command= SQLCOM_END; /* to avoid confusing VIEW detectors */
   session->set_time();
   session->query_id= query_id.value();
@@ -205,6 +209,9 @@
   {
     if (! session->readAndStoreQuery(packet, packet_length))
       break;					// fatal error is set
+    DRIZZLE_QUERY_START(session->query,
+                        session->thread_id,
+                        const_cast<const char *>(session->db ? session->db : ""));
     const char* end_of_stmt= NULL;
 
     mysql_parse(session, session->query, session->query_length, &end_of_stmt);
@@ -295,15 +302,25 @@
 
   /* Store temp state for processlist */
   session->set_proc_info("cleaning up");
-  session->command=COM_SLEEP;
+  session->command= COM_SLEEP;
   memset(session->process_list_info, 0, PROCESS_LIST_WIDTH);
-  session->query=0;
-  session->query_length=0;
+  session->query= 0;
+  session->query_length= 0;
 
   session->set_proc_info(NULL);
   session->packet.shrink(session->variables.net_buffer_length);	// Reclaim some memory
   free_root(session->mem_root,MYF(MY_KEEP_PREALLOC));
-  return(error);
+
+  if (DRIZZLE_QUERY_DONE_ENABLED() || DRIZZLE_COMMAND_DONE_ENABLED())
+  {
+    if (command == COM_QUERY)
+    {
+      DRIZZLE_QUERY_DONE(session->is_error());
+    }
+    DRIZZLE_COMMAND_DONE(session->is_error());
+  }
+
+  return error;
 }
 
 
@@ -759,8 +776,12 @@
           if (*found_semicolon &&
               (session->query_length= (ulong)(*found_semicolon - session->query)))
             session->query_length--;
+          DRIZZLE_QUERY_EXEC_START(session->query,
+                                   session->thread_id,
+                                   const_cast<const char *>(session->db ? session->db : ""));
           /* Actually execute the query */
           mysql_execute_command(session);
+          DRIZZLE_QUERY_EXEC_DONE(0);
 	}
       }
     }
@@ -1819,6 +1840,8 @@
 {
   assert(session->m_lip == NULL);
 
+  DRIZZLE_QUERY_PARSE_START(session->query);
+
   /* Set Lex_input_stream. */
 
   session->m_lip= lip;
@@ -1835,6 +1858,8 @@
 
   session->m_lip= NULL;
 
+  DRIZZLE_QUERY_PARSE_DONE(mysql_parse_status || session->is_fatal_error);
+
   /* That's it. */
 
   return mysql_parse_status || session->is_fatal_error;

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-09-16 20:58:28 +0000
+++ drizzled/sql_select.cc	2009-09-21 14:34:41 +0000
@@ -124,7 +124,7 @@
 {
   bool res;
   register Select_Lex *select_lex= &lex->select_lex;
-  DRIZZLE_SELECT_START();
+  DRIZZLE_SELECT_START(session->query);
 
   if (select_lex->master_unit()->is_union() ||
       select_lex->master_unit()->fake_select_lex)
@@ -157,8 +157,8 @@
   if (unlikely(res))
     result->abort();
 
-  DRIZZLE_SELECT_END();
-  return(res);
+  DRIZZLE_SELECT_DONE(res, session->limit_found_rows);
+  return res;
 }
 
 /*

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2009-08-27 18:06:03 +0000
+++ drizzled/sql_update.cc	2009-09-23 17:01:21 +0000
@@ -139,9 +139,12 @@
   List<Item> all_fields;
   Session::killed_state killed_status= Session::NOT_KILLED;
 
-  DRIZZLE_UPDATE_START();
+  DRIZZLE_UPDATE_START(session->query);
   if (session->openTablesLock(table_list))
+  {
+    DRIZZLE_UPDATE_DONE(1, 0, 0);
     return 1;
+  }
 
   session->set_proc_info("init");
   table= table_list->table;
@@ -179,8 +182,8 @@
   if (select_lex->inner_refs_list.elements &&
     fix_inner_refs(session, all_fields, select_lex, select_lex->ref_pointer_array))
   {
-    DRIZZLE_UPDATE_END();
-    return(-1);
+    DRIZZLE_UPDATE_DONE(1, 0, 0);
+    return -1;
   }
 
   if (conds)
@@ -215,9 +218,9 @@
     free_underlaid_joins(session, select_lex);
     if (error)
       goto abort;				// Error in where
-    DRIZZLE_UPDATE_END();
+    DRIZZLE_UPDATE_DONE(0, 0, 0);
     session->my_ok();				// No matching records
-    return(0);
+    return 0;
   }
   if (!select && limit != HA_POS_ERROR)
   {
@@ -627,7 +630,6 @@
   id= session->arg_of_last_insert_id_function ?
     session->first_successful_insert_id_in_prev_stmt : 0;
 
-  DRIZZLE_UPDATE_END();
   if (error < 0)
   {
     char buff[STRING_BUFFER_USUAL_SIZE];
@@ -638,7 +640,8 @@
   }
   session->count_cuted_fields= CHECK_FIELD_IGNORE;		/* calc cuted fields */
   session->abort_on_warning= 0;
-  return((error >= 0 || session->is_error()) ? 1 : 0);
+  DRIZZLE_UPDATE_DONE((error >= 0 || session->is_error()), found, updated);
+  return ((error >= 0 || session->is_error()) ? 1 : 0);
 
 err:
   delete select;
@@ -651,8 +654,8 @@
   session->abort_on_warning= 0;
 
 abort:
-  DRIZZLE_UPDATE_END();
-  return(1);
+  DRIZZLE_UPDATE_DONE(1, 0, 0);
+  return 1;
 }
 
 /*

=== modified file 'drizzled/statement/delete.cc'
--- drizzled/statement/delete.cc	2009-08-10 18:03:39 +0000
+++ drizzled/statement/delete.cc	2009-09-13 04:30:14 +0000
@@ -22,12 +22,14 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/lock.h>
+#include <drizzled/probes.h>
 #include <drizzled/statement/delete.h>
 
 using namespace drizzled;
 
 bool statement::Delete::execute()
 {
+  DRIZZLE_DELETE_START(session->query);
   TableList *first_table= (TableList *) session->lex->select_lex.table_list.first;
   TableList *all_tables= session->lex->query_tables;
   Select_Lex *select_lex= &session->lex->select_lex;

=== modified file 'drizzled/statement/insert.cc'
--- drizzled/statement/insert.cc	2009-08-26 23:47:28 +0000
+++ drizzled/statement/insert.cc	2009-09-13 04:30:14 +0000
@@ -22,6 +22,7 @@
 #include <drizzled/show.h>
 #include <drizzled/lock.h>
 #include <drizzled/session.h>
+#include <drizzled/probes.h>
 #include <drizzled/statement/insert.h>
 
 using namespace drizzled;
@@ -43,6 +44,8 @@
     return true;
   }
 
+  DRIZZLE_INSERT_START(session->query);
+
   bool res= mysql_insert(session, 
                          all_tables, 
                          session->lex->field_list, 

=== modified file 'drizzled/statement/insert_select.cc'
--- drizzled/statement/insert_select.cc	2009-08-27 00:39:15 +0000
+++ drizzled/statement/insert_select.cc	2009-09-13 04:30:14 +0000
@@ -22,6 +22,7 @@
 #include <drizzled/show.h>
 #include <drizzled/lock.h>
 #include <drizzled/session.h>
+#include <drizzled/probes.h>
 #include <drizzled/statement/insert_select.h>
 
 using namespace drizzled;
@@ -54,6 +55,7 @@
 
   if (! (res= session->openTablesLock(all_tables)))
   {
+    DRIZZLE_INSERT_SELECT_START(session->query);
     /* Skip first table, which is the table we are inserting in */
     TableList *second_table= first_table->next_local;
     select_lex->table_list.first= (unsigned char*) second_table;

=== modified file 'mysys/my_bitmap.cc'
--- mysys/my_bitmap.cc	2009-07-30 02:39:13 +0000
+++ mysys/my_bitmap.cc	2009-09-15 01:32:36 +0000
@@ -29,7 +29,7 @@
     Create a mask with the upper 'unused' bits set and the lower 'used'
     bits clear. The bits within each byte is stored in big-endian order.
    */
-  unsigned char const mask= (~((1 << used) - 1)) & 255;
+  unsigned char const mask= static_cast<unsigned char const>((~((1 << used) - 1)) & 255); 
 
   /*
     The first bytes are to be set to zero since they represent real  bits
@@ -87,9 +87,9 @@
 bool MyBitmap::testAndSet(const uint32_t bitPos)
 {
   unsigned char *value= ((unsigned char*) bitmap) + (bitPos / 8);
-  unsigned char bit= 1 << ((bitPos) & 7);
-  unsigned char res= (*value) & bit;
-  *value|= bit;
+  unsigned char bit= static_cast<unsigned char>(1 << ((bitPos) & 7));
+  unsigned char res= static_cast<unsigned char>((*value) & bit);
+  *value= static_cast<unsigned char>(*value | bit);
   return res;
 }
 
@@ -98,9 +98,9 @@
 bool MyBitmap::testAndClear(const uint32_t bitPos)
 {
   unsigned char *byte= (unsigned char*) bitmap + (bitPos / 8);
-  unsigned char bit= 1 << ((bitPos) & 7);
-  unsigned char res= (*byte) & bit;
-  *byte&= ~bit;
+  unsigned char bit= static_cast<unsigned char>(1 << ((bitPos) & 7));
+  unsigned char res= static_cast<unsigned char>((*byte) & bit);
+  *byte= static_cast<unsigned char>(*byte & ~bit);
   return res;
 }
 
@@ -133,7 +133,7 @@
   m+= prefix_bytes;
   if ((prefix_bits= prefix_size & 7))
   {
-    *m++= (1 << prefix_bits)-1;
+    *m++= static_cast<unsigned char>((1 << prefix_bits)-1);
   }
   if ((d= numOfBytesInMap() - prefix_bytes))
   {
@@ -284,7 +284,7 @@
 
 void MyBitmap::setAbove(const uint32_t from_byte, const uint32_t use_bit)
 {
-  unsigned char use_byte= use_bit ? 0xff : 0;
+  unsigned char use_byte= static_cast<unsigned char>(use_bit ? 0xff : 0);
   unsigned char *to= (unsigned char *) bitmap + from_byte;
   unsigned char *end= (unsigned char *) bitmap + (n_bits+7)/8;
 

=== modified file 'mysys/my_bitmap.h'
--- mysys/my_bitmap.h	2009-07-30 04:00:24 +0000
+++ mysys/my_bitmap.h	2009-09-15 01:32:36 +0000
@@ -171,7 +171,10 @@
    */
   void setBit(const uint32_t bit)
   {
-    ((unsigned char *)bitmap)[bit / 8] |= (1 << ((bit) & 7));
+    reinterpret_cast<unsigned char *>(bitmap)[bit / 8]= 
+      static_cast<unsigned char>(
+      (reinterpret_cast<unsigned char *>(bitmap))[bit / 8] | 
+      (1 << ((bit) & 7)));
   }
 
   /**
@@ -181,7 +184,10 @@
    */
   void flipBit(const uint32_t bit)
   {
-    ((unsigned char *)bitmap)[bit / 8] ^= (1 << ((bit) & 7));
+    reinterpret_cast<unsigned char *>(bitmap)[bit / 8]= 
+      static_cast<unsigned char>(
+      (reinterpret_cast<unsigned char *>(bitmap))[bit / 8] ^ 
+      (1 << ((bit) & 7)));
   }
 
   /**
@@ -191,7 +197,10 @@
    */
   void clearBit(const uint32_t bit)
   {
-    ((unsigned char *)bitmap)[bit / 8] &= ~ (1 << ((bit) & 7));
+    reinterpret_cast<unsigned char *>(bitmap)[bit / 8]= 
+      static_cast<unsigned char>(
+      (reinterpret_cast<unsigned char *>(bitmap))[bit / 8] & 
+      ~ (1 << ((bit) & 7)));
   }
 
   /**

=== modified file 'mysys/my_copy.cc'
--- mysys/my_copy.cc	2009-04-17 21:01:47 +0000
+++ mysys/my_copy.cc	2009-09-15 01:32:36 +0000
@@ -79,7 +79,8 @@
 			     MyFlags)) < 0)
       goto err;
 
-    while ((Count=my_read(from_file, buff, sizeof(buff), MyFlags)) != 0)
+    while ((Count= static_cast<uint32_t>(my_read(from_file, buff,
+            sizeof(buff), MyFlags))) != 0)
     {
 	if (Count == (uint32_t) -1 ||
 	    my_write(to_file,buff,Count,MYF(MyFlags | MY_NABP)))

=== modified file 'mysys/my_getopt.cc'
--- mysys/my_getopt.cc	2009-06-24 23:48:15 +0000
+++ mysys/my_getopt.cc	2009-09-15 01:32:36 +0000
@@ -217,7 +217,7 @@
         }
 	else
         {
-          length= strlen(opt_str);
+          length= static_cast<uint32_t>(strlen(opt_str));
 	  optend= 0;
         }
 

=== modified file 'mysys/my_init.cc'
--- mysys/my_init.cc	2009-05-04 18:11:56 +0000
+++ mysys/my_init.cc	2009-09-15 01:32:36 +0000
@@ -121,10 +121,10 @@
 Non-physical pagefaults %ld, Physical pagefaults %ld, Swaps %ld\n\
 Blocks in %ld out %ld, Messages in %ld out %ld, Signals %ld\n\
 Voluntary context switches %ld, Involuntary context switches %ld\n",
-	      (rus.ru_utime.tv_sec * SCALE_SEC +
-	       rus.ru_utime.tv_usec / SCALE_USEC) / 100.0,
-	      (rus.ru_stime.tv_sec * SCALE_SEC +
-	       rus.ru_stime.tv_usec / SCALE_USEC) / 100.0,
+	      static_cast<double>((rus.ru_utime.tv_sec * SCALE_SEC +
+	      rus.ru_utime.tv_usec / SCALE_USEC)) / 100.0,
+	      static_cast<double>((rus.ru_stime.tv_sec * SCALE_SEC +
+	      rus.ru_stime.tv_usec / SCALE_USEC)) / 100.0,
 	      rus.ru_maxrss, rus.ru_idrss,
 	      rus.ru_minflt, rus.ru_majflt,
 	      rus.ru_nswap, rus.ru_inblock, rus.ru_oublock,

=== modified file 'mysys/my_lib.cc'
--- mysys/my_lib.cc	2009-05-23 16:15:00 +0000
+++ mysys/my_lib.cc	2009-09-15 01:32:36 +0000
@@ -142,7 +142,7 @@
   (void) closedir(dirp);
 
   result->dir_entry= (FILEINFO *)dir_entries_storage->buffer;
-  result->number_off_files= dir_entries_storage->elements;
+  result->number_off_files= static_cast<uint>(dir_entries_storage->elements);
 
   if (!(MyFlags & MY_DONT_SORT))
     my_qsort((void *) result->dir_entry, result->number_off_files,

=== modified file 'mysys/my_symlink2.cc'
--- mysys/my_symlink2.cc	2009-06-16 01:36:01 +0000
+++ mysys/my_symlink2.cc	2009-09-15 01:32:36 +0000
@@ -134,7 +134,8 @@
   char link_name[FN_REFLEN], tmp_name[FN_REFLEN];
   int sym_link_size= -1;
   int was_symlink= (!my_disable_symlinks &&
-                   (sym_link_size= readlink(from,link_name,FN_REFLEN-1)) != -1);
+                   (sym_link_size= static_cast<int>(readlink(from,link_name,
+                                                    FN_REFLEN-1))) != -1); 
   int result=0;
   int name_is_different;
 

